require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/artifact_state_template/artifact_state_template_attributes'
require 'uu/os/artifact_state_template/artifact_state_template_create'
require 'uu/os/artifact_state_template/artifact_state_template_get_artifact_state_template_list'
require 'uu/os/artifact_state_template/artifact_state_template_set_attributes'


# Module Unicorn Universe.
module UU
  # Module Operating System.
  module OS
    # Module Artifact State Template.
    module ArtifactStateTemplate
      extend self

      # Service path
      PATH = 'uu/os/ArtifactStateTemplate'

      # Creates a new artifact state template on the specified meta artifact.
      #
      # @param [String, UU::OS::UESURI] meta_artifact_uri <b>Not nil.</b> URI of the meta artifact the new artifact state template is to be created on.
      # @param [UU::OS::ArtifactStateTemplate::ArtifactStateTemplateCreate] options Specific options.
      # @option options [String] :name <b>Not nil.</b> Name of the new artifact state template.
      # @option options [String] :code Code of the new artifact state template. If it is not specified, the code is generated automatically using the system sequence. The code must be unique in the scope of the given meta artifact, otherwise the create command fails.
      # @option options [String] :description Description of the new artifact state template. If it is not specified, it is left empty.
      # @option options [String, UU::OS::Artifact::ArtifactStateType] :type <b>Not nil.</b> Type of the new artifact state template. All possible types can be found as constants in {UU::OS::Artifact::ArtifactStateType} class.
      # @option options [Boolean] :entry_state After an artifact is created from the related meta artifact, it will be set into the state marked as the entry state. There is always just one artifact state template marked as the entry state on every meta artifact. By setting true, the new artifact state template will become the entry state instead of the one previously specified. The default value is false.
      # @option options [String, UU::OS::UESURI] :icon_uri Icon of the new artifact state template. If not specified, there will be used a default icon derived from the specified type. Allowed format of icon: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'.
      # @option options [Integer] :order Order of the new artifact state template. All artifact state templates are suggested to the user in this order when setting the artifact state to an artifact which is created according to the given meta artifact. If it is not set, no specific order is specified.
      # @option options [Boolean] :create_activity_template By setting this parameter to true, the user ensures that an activity for setting this state to artifacts will be created after the creation of the artifact state template is finished. The existence of such an activity will enable setting this state to artifacts. The default value is true.
      # 
      # @return [UU::OS::UESURI] URI of the new artifact state template.
      # 
      # @example
      #   # Create an artifact state template
      #   UU::OS::ArtifactStateTemplate.create('ues:TERRITORY:META_ARTIFACT',
      #      :name => 'New Artifact State Template Name',
      #      :type => 'FINAL',
      #      :icon_uri => 'ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{S00_C01}',
      #      :order => 120)
      def create(meta_artifact_uri, options)
        svc = UU::OS::REST::RemoteClient.new(ArtifactStateTemplate)
        payload = UU::OS::ArtifactStateTemplate::ArtifactStateTemplateCreate.new(options).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', meta_artifact_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns attributes of the specified artifact state template.
      # 
      # @param [String, UU::OS::UESURI] artifact_state_template_uri <b>Not nil.</b> URI of the artifact state template whose attributes are to be returned.
      # 
      # @return [UU::OS::ArtifactStateTemplate::ArtifactStateTemplateAttributes] Object with attributes of the artifact state template.
      # 
      # @example
      #   # Get attributes of an artifact state template
      #   UU::OS::ArtifactStateTemplate.get_attributes('ues:TERRITORY:META_ARTIFACT:ARTIFACT_STATE_TEMPLATE')
      def get_attributes(artifact_state_template_uri)
        svc = UU::OS::REST::RemoteClient.new(ArtifactStateTemplate)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', artifact_state_template_uri)
          return ArtifactStateTemplate::ArtifactStateTemplateAttributes.new(res)
        end
      end

      # Sets attributes of an artifact state template. This command cannot modify the type and activity template.
      #
      # @param [String, UU::OS::UESURI] artifact_state_template_uri <b>Not nil.</b> URI of the artifact state template whose attributes are to be modified.
      # @param [UU::OS::ArtifactStateTemplate::ArtifactStateTemplateSetAttributes] options Specific options.
      # @option options [String] :name Name of the new artifact state template. If not specified, the create command fails.
      # @option options [String] :code Code of the new artifact state template. If it is not specified, the code is generated automatically using the system sequence. The code must be unique in the scope of the given meta artifact, otherwise the create command fails.
      # @option options [String] :description Description of the new artifact state template. If it is not specified, it is left empty.
      # @option options [Boolean] :entry_state After an artifact is created from the related meta artifact, it will be set into the state marked as the entry state. There is always just one artifact state template marked as the entry state on every meta artifact. By setting true, the new artifact state template will become the entry state instead of the one previously specified. The default value is false.
      # @option options [String, UU::OS::UESURI] :icon_uri URI of the new artifact state template's icon. If it is not specified, the icon is not modified. If nil URI ('ues:{[-1]}:{[-1]}:') is set, the artifact state template's icon is replaced by a default icon derived from the type. Allowed format of icon: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'.
      # @option options [Integer] :order Order of the new artifact state template. All artifact state templates are suggested to the user in this order when setting the artifact state to an artifact which is created according to the given meta artifact. If it is not specified, the parameter is not modified.
      # 
      # @return [UU::OS::UESURI] URI of the modified artifact state template.
      # 
      # @example
      #   # Sets a few attributes of an artifact state template
      #   UU::OS::ArtifactStateTemplate.set_attributes('ues:TERRITORY:META_ARTIFACT:ARTIFACT_STATE_TEMPLATE',
      #      :name => 'New Artifact State Template Name', :entry_state => true, order => 95,
      #      :icon_uri => 'ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{S00_C01}')
      def set_attributes(artifact_state_template_uri, options)
        svc = UU::OS::REST::RemoteClient.new(ArtifactStateTemplate)
        payload = UU::OS::ArtifactStateTemplate::ArtifactStateTemplateSetAttributes.new(options).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', artifact_state_template_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Deletes the specified artifact state template from the system. It cannot be deleted in case there is an action
      # that sets a state based on this template. Also, a system state and any state set as entry state cannot be deleted.
      #
      # @param [String, UU::OS::UESURI] artifact_state_template_uri <b>Not nil.</b> URI of the artifact state template which is to be deleted.
      # 
      # @example
      #   # Deletes an artifact state template
      #   UU::OS::ArtifactStateTemplate.delete('ues:TERRITORY:META_ARTIFACT:ARTIFACT_STATE_TEMPLATE')
      def delete(artifact_state_template_uri)
        svc = UU::OS::REST::RemoteClient.new(ArtifactStateTemplate)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('delete', artifact_state_template_uri)
        end
      end

      # Returns a list of artifact state templates on the specified meta artifact.
      # 
      # @param [String, UU::OS::UESURI] meta_artifact_uri <b>Not nil.</b> URI of the meta artifact whose artifact state templates are to be listed.
      # @param [UU::OS::ArtifactStateTemplate::ArtifactStateTemplateGetArtifactStateTemplateList] criteria Specific options.
      # @option criteria [String] :query Query for filtering and ordering of the result list. It has to conform the ues query syntax (see the documentation below for more info). If it is not specified, no filtering is applied and entries are ordered by the parameter order and name as the second criteria. Otherwise, it is possible to filter and sort the result list by the following attributes: name, code, type, entryState, order, activityTemplateExists.
      #
      # @see https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation.
      #
      # @return [UU::OS::REST::ResultList<ArtifactStateTemplate::ArtifactStateTemplateGetArtifactStateTemplateList, ArtifactStateTemplate::ArtifactStateTemplateAttributes>] List of artifact state templates according to the entered entity. Contains instances of {UU::OS::ArtifactStateTemplate::ArtifactStateTemplateAttributes}.
      #
      # @example
      #   # Returns list of artifact state templates which are not the entry state and have an active type
      #   UU::OS::ArtifactStateTemplate.get_artifact_state_template_list('ues:TERRITORY:META_ARTIFACT',
      #      :query => 'type="ACTIVE" AND entry_state=false')
      def get_artifact_state_template_list(meta_artifact_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(ArtifactStateTemplate)
        dto = ArtifactStateTemplate::ArtifactStateTemplateGetArtifactStateTemplateList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getArtifactStateTemplateList', meta_artifact_uri)
          return UU::OS::REST::ResultList.new(ArtifactStateTemplate::ArtifactStateTemplateGetArtifactStateTemplateList, ArtifactStateTemplate::ArtifactStateTemplateAttributes, res)
        end
      end

    end
  end

end
