require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/meta_permission/meta_permission_create_mask'
require 'uu/os/meta_permission/meta_permission_get_authority_data'
require 'uu/os/meta_permission/meta_permission_get_authority_list_item'
require 'uu/os/meta_permission/meta_permission_get_authority_list'
require 'uu/os/meta_permission/meta_permission_get_mask_attributes'
require 'uu/os/meta_permission/meta_permission_get_mask_data'
require 'uu/os/meta_permission/meta_permission_get_mask_list'
require 'uu/os/meta_permission/meta_permission_get_system_group_attributes'
require 'uu/os/meta_permission/meta_permission_get_system_group_list'
require 'uu/os/meta_permission/meta_permission_grant'
require 'uu/os/meta_permission/meta_permission_revoke'
require 'uu/os/meta_permission/meta_permission_set_mask_attributes'
require 'uu/os/meta_permission/meta_permission_set_mask_data'
require 'base64'

# Module Unicorn Universe.
module UU
  # Module Operating System.
  module OS
    # This service allows managing meta permissions and permission masks for template objects (only meta artifacts in current version).
    # Meta permissions provide access to Use Cases of objects (Artifact, Property, ...) created according to their template objects.
    # Permission masks allow to prescribe permission settings on objects. For more information see {uuOS Access Rights - Guideline}[https://plus4u.net/ues/sesm?SessFree=ues%253A%255B210795%255D%253A%255B44191580132632900%255D%253A].
    module MetaPermission
      extend self

      # REST service url path.
      PATH = 'ues/core/security/UESMetaPermission'
      # Grants meta permissions on a template object (only meta artifacts in current version) for given Use Cases.
      # It is possible to grant meta permissions on template objects for any allowed authority (Role, Group or System Group). <br>
      # To list all allowed system groups use uuCommand {#get_system_group_list}. <br>
      # All possible levels may be found as constants in {UU::OS::MetaPermission::MetaPermissionLevel} class.
      #
      # @param [String, UU::OS::UESURI] template_uri UESURI of the template object for which meta permissions will be granted. Must not be nil.
      # @param [UU::OS::MetaPermission::MetaPermissionGrant] meta_permission_grant Specific options.
      # @option meta_permission_grant [String, UU::OS::UESURI] :authority_uri UESURI of the authority for which the meta permissions will be granted. Must not be nil.
      # @option meta_permission_grant [String, File, UU::OS::REST::BinaryValue] :data Information about meta permission settings in JSON format. The meta permission is specified by the use case and the level. Must not be nil.
      #
      #
      # @example
      #   # Prepare data for the meta permission in JSON format.
      #   grant_data = '[{"useCaseCode":"AU_103129","castLevel":"$GUEST"}]'
      #
      #   # Grant meta permissions for Group or Role.
      #   UU::OS::MetaPermission.grant('ues:TERRITORY:META_ARTIFACT', :authority_uri => 'ues:TERRITORY:ROLE', :data => grant_data)
      #
      #   # Grant meta permissions for System Group.
      #   UU::OS::MetaPermission.grant('ues:TERRITORY:META_ARTIFACT', :authority_uri => 'ues:${system}:COMPETENT_FOR_ART_IFC', :data => grant_data)
      def grant(template_uri, meta_permission_grant)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)
        payload = UU::OS::MetaPermission::MetaPermissionGrant.new(meta_permission_grant).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('grant', template_uri, payload)
        end
      end

      # Revokes meta permissions on a template object (only meta artifacts in current version) for given Use Cases.
      # It is possible to revoke meta permissions on template objects for any allowed authority (Role, Group or System Group). <br>
      # To list all allowed system groups use uuCommand {#get_system_group_list}.
      #
      # @param [String, UU::OS::UESURI] template_uri UESURI of the template object for which the meta permissions will be revoked. Must not be nil.
      # @param [UU::OS::MetaPermission::MetaPermissionRevoke] meta_permission_revoke Specific options.
      # @option meta_permission_revoke [String, UU::OS::UESURI] :authority_uri UESURI of the authority for which the meta permissions will be revoked. Must not be nil.
      # @option meta_permission_revoke [String, File, UU::OS::REST::BinaryValue] :data List of Use Cases (in JSON format) for which the meta permission will be revoked.
      # @option meta_permission_revoke [Boolean] :all When true is set, meta permissions will be revoked for all Use Cases. Default value is false. Option :data has higher priority than this one.
      #
      # @example
      #   # Revoke meta permissions on all Use Cases for System Group
      #   UU::OS::MetaPermission.revoke('ues:TERRITORY:META_ARTIFACT', :authority_uri => 'ues:${system}:COMPETENT_FOR_ART_IFC', :all => true)
      #
      #   # Revoke meta permissions on all Use Cases for Group or Role.
      #   UU::OS::MetaPermission.revoke('ues:TERRITORY:META_ARTIFACT', :authority_uri => 'ues:TERRITORY:ROLE', :all => true)
      #
      #   # Revoke meta permissions on specified Use Cases for Group or Role.
      #   revoke_data = '[{"useCaseCode":"AU_103129"},{"useCaseCode":"AU_103300"}]'
      #   UU::OS::MetaPermission.revoke('ues:TERRITORY:META_ARTIFACT', :authority_uri => 'ues:TERRITORY:ROLE', :data => revoke_data)
      def revoke(template_uri, meta_permission_revoke)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)
        payload = UU::OS::MetaPermission::MetaPermissionRevoke.new(meta_permission_revoke).to_json        

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('revoke', template_uri, payload)
        end
      end

      # Returns a list of authorities (Roles, Groups or System Groups) who have been granted meta permissions on a template object (only meta artifacts in current version).
      #
      # @param [String, UU::OS::UESURI] template_uri UESURI of the template object for which to list all authorities with granted meta permissions.
      # @param [UU::OS::MetaPermission::MetaPermissionGetAuthorityList] criteria Specific options.
      # @option criteria [String] :query Query for filtering and ordering of the result list (more information in {uuQuery - Guideline}[https://plus4u.net/ues/sesm?SessFree=ues%253AUNI-BT%255B210795%255D%253AUES.DOC%252FCMD_V1%252FUESQUERY%255B44191574519623267%255D%253A]). It is possible to filter the result list by authorityUri and authorityName.
      #
      # @return [Array] An Array with the command result. Contains instances of {UU::OS::MetaPermission::MetaPermissionGetAuthorityListItem}.
      #
      # @example
      #   # Get a list of all authorities which have been granted meta permissions for a meta artifact.
      #   UU::OS::MetaPermission.get_authority_list('ues:TERRITORY:META_ARTIFACT')
      #
      #   # Get a list of all authorities with authority_name that starts with string John.
      #   UU::OS::MetaPermission.get_authority_list('ues:TERRITORY:META_ARTIFACT', :query => "authority_name='John%'")
      def get_authority_list(template_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)
        dto = MetaPermission::MetaPermissionGetAuthorityList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAuthorityList', template_uri)
          return UU::OS::REST::ResultList.new(MetaPermission::MetaPermissionGetAuthorityList, MetaPermission::MetaPermissionGetAuthorityListItem, res)
        end
      end

      # Returns data about meta permissions granted to the given authority (Roles, Groups or System Groups) on a template object (only meta artifacts in current version). <br>
      # To list all authorities with granted meta permissions use uuCommand {#get_authority_list}.
      #
      # @param [String, UU::OS::UESURI] template_uri UESURI of the template object for which the data about meta permission settings will be returned.
      # @param [UU::OS::MetaPermission::MetaPermissionGetAuthorityData] options Specific options.
      # @option options [String, UU::OS::UESURI] :authority_uri UESURI of the authority whose granted meta permission settings will be returned. Must not be nil.
      # @option options [String] :requested_mime_type MIME type of the returned value. Possible values are 'application/json' for JSON and 'text/csv' for CSV. JSON is returned by default.
      #
      # @return [UU::OS::REST::BinaryValue] Data about meta permission settings for given template and authority.
      #
      # @example
      #   # Get meta permissions granted to Group or Role in JSON format.
      #   UU::OS::MetaPermission.get_authority_data('ues:TERRITORY:META_ARTIFACT', :authority_uri => 'ues:TERRITORY:ROLE')
      #
      #   # Get meta permissions granted to System Group in JSON format.
      #   UU::OS::MetaPermission.get_authority_data('ues:TERRITORY:META_ARTIFACT', :authority_uri => 'ues:${system}:COMPETENT_FOR_ART_IFC')
      #
      #   # Get meta permissions granted to Group or Role in CSV format.
      #   UU::OS::MetaPermission.get_authority_data('ues:TERRITORY:META_ARTIFACT', :authority_uri => 'ues:TERRITORY:ROLE', :requested_mime_type => 'text/csv')
      def get_authority_data(template_uri, options)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)
        dto = UU::OS::MetaPermission::MetaPermissionGetAuthorityData.new(options)
        if dto.requested_mime_type
          svc.add_parameter('requestedMimeType', dto.requested_mime_type)
        end
        svc.add_parameter('authorityUri', dto.authority_uri)

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAuthorityData', template_uri)
          return UU::OS::REST::BinaryValue.new(res, true)
        end
      end

      # Creates a permission mask. (Mask contains prescription of the levels of meta permissions.)
      # @param [String, UU::OS::UESURI] template_uri UESURI of the template object for which permission mask will be created.
      # @param [UU::OS::MetaPermission::MetaPermissionCreateMask] mask Specific options.
      # @option mask [String] :name Name of the new permission mask. Must not be nil.
      # @option mask [String] :code Code of the new permission mask. Must be unique for the template object. When nil is set, new code is generated automatically from the sequence.
      # @option mask [String] :description Description of the new permission mask.
      # @option mask [String, File, UU::OS::REST::BinaryValue] :data Data settings of the new permission mask in JSON format (containing prescription of use cases and levels).
      # @option mask [Boolean] :default_mask Flag indicating whether the new permission mask is to be set as default for the template object. Default value is false.
      # @option mask [String] :labels Name of the new permission mask in different languages. This attribute must be entered in JSON format.
      #
      # @return [UU::OS::UESURI] UESURI of the new permission mask.
      #
      # @example
      #   # Prepare new permission mask data.
      #   mask_data = '[{"useCaseCode":"AU_103129","castLevel":"$GUEST"}]'
      #
      #   # Prepare new permission mask labels.
      #   mask_labels = '{"en":"Read","cs":"Čtení","ru":"Чтение"}'
      #
      #   # Create a new permission mask.
      #   UU::OS::MetaPermission.create_mask('ues:TERRITORY:META_ARTIFACT', :name => 'Read', :code => 'READ', :description => 'Read permission mask', :data => mask_data, labels => mask_labels)
      def create_mask(template_uri, mask)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)
        payload = UU::OS::MetaPermission::MetaPermissionCreateMask.new(mask).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('createMask', template_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Deletes a permission mask. It is not possible to delete permission mask which is marked as default.
      # @param [String, UU::OS::UESURI] mask_uri UESURI of the permission mask which will be deleted.
      #
      # @example
      #   # Delete a permission mask.
      #   UU::OS::MetaPermission.delete('ues:TERRITORY:META_ARTIFACT:MASK')
      def delete_mask(mask_uri)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('deleteMask', mask_uri)
        end
      end

      # Returns attributes of a permission mask.
      #
      # @param [String, UU::OS::UESURI] mask_uri UESURI of the permission mask whose attributes will be returned.
      #
      # @return [UU::OS::MetaPermission::MetaPermissionGetMaskAttributes] Attributes of the permission mask.
      #
      # @example
      #   #Get attributes of a permission mask
      #   UU::OS::MetaPermission.get_mask_attributes('ues:TERRITORY:META_ARTIFACT:MASK')
      def get_mask_attributes(mask_uri)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getMaskAttributes', mask_uri)
          return MetaPermission::MetaPermissionGetMaskAttributes.new(res)
        end
      end

      # Returns data settings of a permission mask.
      #
      # @param [String, UU::OS::UESURI] mask_uri UESURI of the permission mask for which the data should be returned.
      # @param [UU::OS::MetaPermission::MetaPermissionGetMaskData] options Specific options.
      # @option options [String] :requested_mime_type  MIME type of the returned value. Possible values are 'application/json' for JSON and 'text/csv' for CSV. JSON is returned by default.
      #
      # @return [UU::OS::REST::BinaryValue] Data settings of the permission mask.
      #
      # @example
      #   UU::OS::MetaPermission.get_mask_data('ues:TERRITORY:META_ARTIFACT:MASK')
      def get_mask_data(mask_uri, options = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)
        dto = UU::OS::MetaPermission::MetaPermissionGetMaskData.new(options)
        if dto.requested_mime_type
          svc.add_parameter('requestedMimeType', dto.requested_mime_type)
        end

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getMaskData', mask_uri)
          return UU::OS::REST::BinaryValue.new(res, true)
        end
      end

      # Returns a list of permission masks for a template object (only meta artifacts in current version).
      #
      # @param [String, UU::OS::UESURI] template_uri UESURI of the template object whose permission masks will be listed.
      # @param [UU::OS::MetaPermission::MetaPermissionGetMaskList] criteria Specific options.
      # @option criteria [String] :query Query for filtering and ordering of the result list (more information in {uuQuery - Guideline}[https://plus4u.net/ues/sesm?SessFree=ues%253AUNI-BT%255B210795%255D%253AUES.DOC%252FCMD_V1%252FUESQUERY%255B44191574519623267%255D%253A]). It is possible to filter the result list by name and code.
      #
      # @return [Array] An Array with the command result. Contains instances of {UU::OS::MetaPermission::MetaPermissionGetMaskAttributes}.
      #
      # @example
      #   # Get a list of all permission masks whose name starts with string "Create" and code starts with string "CREATE".
      #   UU::OS::MetaPermission.get_mask_list('ues:TERRITORY:META_ARTIFACT', :query => "name='Create%' AND code='CREATE%'")
      def get_mask_list(template_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)
        dto = MetaPermission::MetaPermissionGetMaskList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getMaskList', template_uri)
          return UU::OS::REST::ResultList.new(MetaPermission::MetaPermissionGetMaskList, MetaPermission::MetaPermissionGetMaskAttributes, res)
        end
      end

      # Modifies attributes of a permission mask.
      #
      # @param [String, UU::OS::UESURI] mask_uri UESURI of the permission mask whose attributes will be changed.
      # @param [UU::OS::MetaPermission::MetaPermissionSetMaskAttributes] mask Specific options.
      # @option mask [String] :name New name of the permission mask. When nil is set, the attribute will not be modified. Must not be blank.
      # @option mask [String] :code New code of the permission mask. Must be unique for the template object. When nil is set, the attribute will not be modified. When blank string is set, a new code will be generated automatically from the sequence.
      # @option mask [String] :description New description of the permission mask.
      # @option mask [Boolean] :default_mask Flag indicating whether the permission mask is set as default for the template for which the mask is defined.
      # @option mask [String] :labels New names of the permission mask in different laguages. This attribute must be entered in JSON format.
      #
      # @return [UU::OS::UESURI] UESURI of the permission mask to be modified.
      #
      # @example
      #   # Prepare labels
      #   mask_labels = '{"en":"Read / Write","cs":"Čtení / Zápis","ru":"Чтение / записи"}'
      #
      #   # Modify a permission mask.
      #   UU::OS::MetaPermission.set_mask_attributes('ues:TERRITORY:META_ARTIFACT:MASK', :name => 'Read / Write', :code => 'READWRITE',
      #     :description => 'Read / Write permission mask', :default_mask => true, :labels => mask_labels)
      def set_mask_attributes(mask_uri, mask = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)
        payload = UU::OS::MetaPermission::MetaPermissionSetMaskAttributes.new(mask).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setMaskAttributes', mask_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Stores a permission mask settings. The mask settings is list of pairs an use case code (access unit in current version)
      # and a cast level.
      #
      # @param [String, UU::OS::UESURI] mask_uri UESURI of the permission mask whose data should be modified.
      # @param [UU::OS::MetaPermission::MetaPermissionSetMaskData] mask_data Specific options.
      # @option mask_data [String] :data Data for settings permission mask. Only JSON ("application/json") or CSV ("text/csv") format is accepted.
      #
      # @example
      #   # Prepare new permission mask data in JSON Format
      #   mask_data = '[{"useCaseCode":"AU_103129","castLevel":"$GUEST"}]'
      #
      #   # Set data of an existing permission mask
      #   UU::OS::MetaPermission.set_mask_data('ues:TERRITORY:META_ARTIFACT:MASK', :data => mask_data)
      def set_mask_data(mask_uri, mask_data)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)
        payload = UU::OS::MetaPermission::MetaPermissionSetMaskData.new(mask_data).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('setMaskData', mask_uri, payload)
        end
      end

      # Returns a list of System Groups of an entity (template object or pack). It may be used to set meta permissions for these groups.
      #
      # @param [String, UU::OS::UESURI] entity_uri UESURI of the template object. Must not be nil.
      #
      # @return [Array] An Array with the command result. Contains instances of {UU::OS::MetaPermission::MetaPermissionGetSystemGroupAttributes}.
      #
      # @example
      #   # Get a list of all System Groups for given entity.
      #   UU::OS::MetaPermission.get_system_group_list('ues:TERRITORY:META_ARTIFACT')
      def get_system_group_list(entity_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaPermission)
        dto = MetaPermission::MetaPermissionGetSystemGroupList.new(criteria)

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getSystemGroupList', entity_uri)
          return UU::OS::REST::ResultList.new(MetaPermission::MetaPermissionGetSystemGroupList, MetaPermission::MetaPermissionGetSystemGroupAttributes, res)
        end
      end

    end
  end
end