require 'uu/os/rest'
require 'uu/os/qos/qos_handler'
require 'uu/os/visual_use_case/visual_use_case_attributes'
require 'uu/os/visual_use_case/visual_use_case_create'
require 'uu/os/visual_use_case/visual_use_case_get_options'
require 'uu/os/visual_use_case/visual_use_case_get_use_case_list_item'
require 'uu/os/visual_use_case/visual_use_case_get_use_case_list'
require 'uu/os/visual_use_case/visual_use_case_set_attributes'
require 'uu/os/visual_use_case/visual_use_case_set_options'
require 'uu/os/visual_use_case/visual_use_case_export'
require 'uu/os/visual_use_case/visual_use_case_get_export_data'
require 'uu/os/visual_use_case/controller_engine'

# Module Unicorn Universe.
module UU
  # Module Operating System.
  module OS
    # Module Visual Use Case.
    module VisualUseCase
      extend self

      # Service path
      PATH = 'uu/os/VisualUseCase'

      # The command creates a visual use case in a specified location. At the very least meta artifact must be specified in
      # {VisualUseCaseCreate} Object. Competent role for the new visual use case will be selected as the most suitable according
      # to specified container and meta artifact (executive/authorized role with connected interface), or can be also specified in
      # {VisualUseCaseCreate} Object.
      #
      # @param [String, UU::OS::UESURI] uri URI of the MetaModelDictionary where the VUC should be created.
      # @param [VisualUseCaseCreate] criteria Object containing attributes for the new VUC.
      # @option criteria [String] :name Name of the new visual use case. If nil is set, default name from meta artifact is used if set, otherwise the create command fails.
      # @option criteria [String] :code Code of the new visual use case. If nil is set, the code is generated automatically using the system sequence. The code must be unique in the given territory, otherwise the create command fails.
      # @option criteria [String] :description Description of the new visual use case. If nil is set, default description from meta artifact is used if set, otherwise no description is set on the new visual use case.
      # @option criteria [String, UU::OS::Artifact::SecurityLevel] :security_level Security level. If nil is set, security level is taken from the meta artifact default security level. If the meta artifact template security level is higher than the user's security clearance, the create command fails.
      # @option criteria [String, UU::OS::UESURI] :icon_uri URI of the visual use case's icon. If nil is set, the role uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option criteria [String] :controller_url_path Path at which the controller will be deployed.
      # @option criteria [String] :controller_class Class to look for when executing this VUC.
      # @option criteria [String, UU::OS::VisualUseCase::ControllerEngine] :controller_engine Engine used for this VUC.
      # @option criteria [String, UU::OS::UESURI] :competent_role_uri URI of the competent role. If nil is set, the executive role from authorization is used. If no role is authorized to create the visual use case into specified container, the create command fails. If the selected role doesn't have connected the role interface with the selected meta artifact, the create command fails. If the selected role doesn't belong to the same organizational unit as a new visual use case or if it doesn't belong to one of superior organizational units, the create command fails.
      # @option criteria [String, UU::OS::UESURI] :meta_artifact_uri URI of the meta artifact. Only a meta artifact with the template class of Visual use case (not its descendants) can be used. If the rights does not permit to create a new visual use case from the meta artifact in the specified location, the create command fails.
      # @option criteria [String, File, UU::OS::REST::BinaryValue] :options Options data for use case calling. Accepted format is JSON.
      # @option criteria [Boolean] :component_content Defines whether the primarily displayed content of the created VisualUseCase is component content. Default value is false.
      #
      # @return [UU::OS::UESURI] URI of the created VUC.
      #
      # @example
      #   # Create visual use case
      #   vucUri = UU::OS::VisualUseCase.create('ues:TERRITORY:APP_PACKAGE', :name => 'New Visual Use Case', :code => 'NEW_VISUAL_USE_CASE',
      #            :controller_engine => UU::OS::VisualUseCase::ControllerEngine::UXML_USECASE_2_0, :controller_class => 'UNI::SimpleValue::Saver::Handle',
      #            :controller_url_path => '/mvc/UNI/SimpleValue/Saver/Handle', :meta_artifact_uri => 'ues:TERRITORY:META_ARTIFACT')
      def create(uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(VisualUseCase)
        payload = UU::OS::VisualUseCase::VisualUseCaseCreate.new(criteria).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # The command deletes the specified visual use case from the system. The command fails when the specified visual use case is
      # referenced by some use case.
      #
      # @param [String, UU::OS::UESURI] uri URI of the VUC to be deleted.
      #
      # @example
      #   # Delete visual use case
      #   UU::OS::VisualUseCase.delete('ues:TERRITORY:VISUAL_USE_CASE')
      def delete(uri)
        svc = UU::OS::REST::RemoteClient.new(VisualUseCase)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', uri)
        end
      end

      # This command returns attributes representing the visual use case specified by the visual_use_case_uri parameter.
      #
      # @param [String, UU::OS::UESURI] visual_use_case_uri URI of the VUC.
      #
      # @return [VisualUseCaseAttributes] Object containing the VUC attributes.
      #
      # @example
      #   # Get visual use case attributes.
      #   attributes = UU::OS::VisualUseCase.get_attributes('ues:TERRITORY:VISUAL_USE_CASE')
      def get_attributes(visual_use_case_uri)
        svc = UU::OS::REST::RemoteClient.new(VisualUseCase)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', visual_use_case_uri)
          return VisualUseCase::VisualUseCaseAttributes.new(res)
        end
      end

      # The command for setting attributes of a specified visual use case. The command can't modify references to object like storage,
      # competent role and so on.
      #
      # @param [String, UU::OS::UESURI] uri URI of the VUC.
      # @param [VisualUseCaseSetAttributes] criteria Object containing new attributes for given VUC.
      # @option criteria [String] :name The new name of the visual use case. If nil is set, the name is not modified.
      # @option criteria [String] :code The new code of the visual use case. If nil is set, the code is not modified.
      # @option criteria [String] :description The new description of the visual use case. If nil is set, the description is not modified.
      # @option criteria [String, UU::OS::Artifact::SecurityLevel] :security_level Security level of the visual use case. If nil is set, the security level is not modified.
      # @option criteria [String, UU::OS::UESURI] :icon_uri URI of the visual use case icon. If nil is set, the icon is not modified. If nil URI ("ues:{[-1]}:{[-1]}:") is set, the visual use case icon is removed and the visual use case uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option criteria [String] :controller_url_path Path at which the controller will be deployed.
      # @option criteria [String] :controller_class Class to look for when executing this VUC.
      # @option criteria [String, UU::OS::VisualUseCase::ControllerEngine] :controller_engine Engine used for this VUC.
      #
      # @return [UU::OS::UESURI] URI of the changed VUC
      #
      # @example
      #   # Set visual use case attributes.
      #   vucUri = UU::OS::VisualUseCase.set_attributes('ues:TERRITORY:VISUAL_USE_CASE', :description => 'Visual Use Case designated for testing purposes',
      #     :name => 'New Visual Use Case', :controller_engine => 'UXML_USECASE_2_0')
      def set_attributes(uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(VisualUseCase)
        payload = UU::OS::VisualUseCase::VisualUseCaseSetAttributes.new(criteria).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns data of the first attachment on the VUC - supposed controller written in ruby.
      #
      # @param [String, UU::OS::UESURI] visual_use_case_uri URI of the VUC.
      #
      # @return [UU::OS::REST::BinaryValue] StreamHandler containing the data.
      #
      # @example
      #   # Get visual use case data.
      #   data = UU::OS::VisualUseCase.get_data('ues:TERRITORY:VISUAL_USE_CASE')
      def get_data(visual_use_case_uri)
        svc = UU::OS::REST::RemoteClient.new(VisualUseCase)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get_binary('getData', visual_use_case_uri)
          return UU::OS::REST::BinaryValue.new(res, true)
        end
      end

      # The command sets options of the specified visual use case.
      #
      # @param [String, UU::OS::UESURI] uri VisualUseCase URI of the VUC.
      # @param [VisualUseCaseSetOptions] criteria Object containing new options for the VUC.
      # @option criteria [String, File, UU::OS::REST::BinaryValue] :data Data for use case options. Only JSON ("application/json") format is accepted.
      #
      # @example
      #   # Set visual use case options.
      #   UU::OS::VisualUseCase.set_options('ues:TERRITORY:VISUAL_USE_CASE', :data => "{\n    \"glossary\": {\n\"title\":\"example ě+ščřěščřžýáíé glossary\",\n\t\t\"GlossDiv\":{\n\"title\":\"S\",\n\t\t\t\"GlossList\":{\n\"GlossEntry\": {\n\"ID\":\"SGML\",\n\t\t\t\t\t\"SortAs\":\"SGML\",\n\t\t\t\t\t\"GlossTerm\":\"Standard Generalized Markup Language\",\n\t\t\t\t\t\"Acronym\":\"SGML\",\n\t\t\t\t\t\"Abbrev\":\"ISO 8879:1986\",\n\t\t\t\t\t\"GlossDef\":{\n\"para\":\"A meta-markup language, used to create markup languages such as DocBook.\",\n\t\t\t\t\t\t\"GlossSeeAlso\":[\"GML\",\"XML\"]\n},\n\t\t\t\t\t\"GlossSee\":\"markup\"\n}\n}\n}\n}\n}")
      def set_options(uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(VisualUseCase)

        payload = UU::OS::VisualUseCase::VisualUseCaseSetOptions.new(criteria).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('setOptions', uri, payload)
        end
      end

      # The command returns options of the specified use case.
      #
      # @param [String, UU::OS::UESURI] uri URI of the VUC.
      # @param [VisualUseCaseGetOptions] criteria Object containing details for the operation.
      #
      # @return [UU::OS::REST::BinaryValue] StreamHandler containing the options.
      #
      # @example
      #   # Get visual use case options.
      #   UU::OS::VisualUseCase.get_options('ues:TERRITORY:VISUAL_USE_CASE')
      def get_options(uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(VisualUseCase)
        options = UU::OS::VisualUseCase::VisualUseCaseGetOptions.new(criteria)
        svc.add_parameter('requestedMimeType', options.requested_mime_type)
        UU::OS::QoS::QoSHandler.auto_retry do
          return UU::OS::REST::BinaryValue.new(svc.get('getOptions', uri), true)
        end
      end

      # Returns list of the use cases that are connected to the specified visual use case. The list of returned use cases could be
      # filtered by the name or code. Returned list does not contain any objects filtered out by executed UESQuery. The list is sorted by the
      # name (and codes in case are equal) of an use case by default.
      #
      # @param [String, UU::OS::UESURI] uri URI of the VUC
      # @param [VisualUseCaseGetUseCaseList] criteria Object containing criteria for listing connected use cases.
      # @option criteria [String] :query The <b>UES query</b> string. It has to conform the ues query syntax (see documentation for more info). Filtering and order of the result list can be specified there. If nil is set, no filtering is applied and entries are ordered by name of referenced artifact (and code and type as the second and the third criteria).
      #
      # See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      #
      # @return [UU::OS::REST::ResultList<VisualUseCase::VisualUseCaseGetUseCaseList, VisualUseCase::VisualUseCaseGetUseCaseListItem>] List of the connected use cases
      #
      # @example
      #   # Get visual use case list.
      #   UU::OS::VisualUseCase.get_use_case_list('ues:TERRITORY:VISUAL_USE_CASE')
      #
      #   #Get a list of visual use cases with name that starts with string Protocol.
      #   UU::OS::VisualUseCase.get_use_case_list('ues:TERRITORY:VISUAL_USE_CASE', :query => "name='Protocol%'")
      def get_use_case_list(uri, criteria=nil)
        svc = UU::OS::REST::RemoteClient.new(VisualUseCase)
        dto = VisualUseCase::VisualUseCaseGetUseCaseList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getUseCaseList', uri)
          return UU::OS::REST::ResultList.new(VisualUseCase::VisualUseCaseGetUseCaseList, VisualUseCase::VisualUseCaseGetUseCaseListItem, res)
        end
      end

      # Exports a visual use case to the XML file, which is saved to the export storage. The export is an asynchronous process.<br>
      # It is possible to wait for the end of the process. More information in {UU::OS::Env::Process Process} and in {UU::OS::REST::Future Future}.
      #
      # @param [String, UU::OS::UESURI] uri URI of the visual use case to export.
      #
      # @return [UU::OS::UESURI] URI of the export process.
      #
      # @example
      #   # Start exporting a visual use case.
      #   process_uri = UU::OS::VisualUseCase.export('ues:TERRITORY:VISUAL_USE_CASE')
      #
      #   # Wait for the end of the process.
      #   export_uri = UU::OS::REST::Future.new(process_uri).get(nil, 600)
      def export(uri)
        svc = UU::OS::REST::RemoteClient.new(VisualUseCase)
        payload = UU::OS::VisualUseCase::VisualUseCaseExport.new(nil).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('export', uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns a XML file generated by the export command {UU::OS::VisualUseCase#export export}.
      #
      # @param [String, UU::OS::UESURI] visual_use_case_uri URI of the visual use case we want to get the exported XML file from.
      # @param [UU::OS::VisualUseCase::VisualUseCaseGetExportData] visual_use_case Specific options.
      # @option visual_use_case [String] :requested_mime_type MIME type of the returned value. All possible MIME types can be found as constants in {UU::OS::Artifact::ExportMimeType ExportMimeType} class. ZIP is returned by default.
      #
      # @return [UU::OS::REST::BinaryValue] Data file of an exported visual use case (XML or ZIP containing XML).
      #
      # @example
      #   # Start exporting a visual use case
      #   process_uri = UU::OS::VisualUseCase.export('ues:TERRITORY:VISUAL_USE_CASE')
      #
      #   # Wait for the end of the process.
      #   export_uri = UU::OS::REST::Future.new(process_uri).get(nil, 600)
      #
      #   # Get exported XML file.
      #   exported_data = UU::OS::VisualUseCase.get_export_data(export_uri, :requested_mime_type => 'application/xml')
      def get_export_data(visual_use_case_uri, visual_use_case = nil)
        svc = UU::OS::REST::RemoteClient.new(VisualUseCase)
        dto = UU::OS::VisualUseCase::VisualUseCaseGetExportData.new(visual_use_case)
        if dto.requested_mime_type
          svc.add_parameter('requestedMimeType', dto.requested_mime_type)
        end
        UU::OS::QoS::QoSHandler.auto_retry do
          return svc.get_binary('getExportData', visual_use_case_uri)
        end
      end

    end

  end
end
